"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


InstrumentType = Union[
    Literal[
        "pan",
        "card_token",
        "redirect",
        "redirect_token",
        "googlepay",
        "applepay",
        "network_token",
        "plaid",
    ],
    UnrecognizedStr,
]
